@@ -4,6 +4,7 @@
  *  Part of the STXXL. See http://stxxl.sourceforge.net
  *
  *  Copyright (C) 2003 Roman Dementiev <dementiev@mpi-sb.mpg.de>
+ *  Copyright (C) 2008 Andreas Beckmann <beckmann@cs.uni-frankfurt.de>
  *
  *  Distributed under the Boost Software License, Version 1.0.
  *  (See accompanying file LICENSE_1_0.txt or copy at
@@ -17,13 +18,14 @@
 // I rework the stable_ksort when I would have a time
 
 
-#include <cmath>
-
 #include <stxxl/bits/mng/mng.h>
 #include <stxxl/bits/mng/buf_istream.h>
 #include <stxxl/bits/mng/buf_ostream.h>
 #include <stxxl/bits/common/simple_vector.h>
 #include <stxxl/bits/algo/intksort.h>
+#include <stxxl/bits/algo/sort_base.h>
+#include <stxxl/bits/common/utils.h>
+
 
 #ifndef STXXL_VERBOSE_STABLE_KSORT
 #define STXXL_VERBOSE_STABLE_KSORT STXXL_VERBOSE1
@@ -131,7 +133,7 @@ namespace stable_ksort_local
     template <typename ExtIterator_>
     void distribute(
         bid_sequence<typename ExtIterator_::vector_type::block_type::bid_type,
-                     typename ExtIterator_::vector_type::alloc_strategy> * bucket_bids,
+                     typename ExtIterator_::vector_type::alloc_strategy_type> * bucket_bids,
         int64 * bucket_sizes,
         const int_type nbuckets,
         const int_type lognbuckets,
@@ -211,6 +213,13 @@ namespace stable_ksort_local
     }
 }
 
+//! \brief Sort records with integer keys
+//! \param first object of model of \c ext_random_access_iterator concept
+//! \param last object of model of \c ext_random_access_iterator concept
+//! \param M amount of memory for internal use (in bytes)
+//! \remark Elements must provide a method key() which returns the integer key.
+//! \remark Not yet fully implemented, it assumes that the keys are uniformly
+//! distributed between [0,(std::numeric_limits<key_type>::max)().
 template <typename ExtIterator_>
 void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
 {
@@ -219,7 +228,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
     typedef typename value_type::key_type key_type;
     typedef typename ExtIterator_::block_type block_type;
     typedef typename block_type::bid_type bid_type;
-    typedef typename ExtIterator_::vector_type::alloc_strategy alloc_strategy;
+    typedef typename ExtIterator_::vector_type::alloc_strategy_type alloc_strategy;
     typedef stable_ksort_local::bid_sequence<bid_type, alloc_strategy> bucket_bids_type;
     typedef stable_ksort_local::type_key<value_type> type_key_;
 
@@ -236,16 +245,15 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
     const unsigned_type ndisks = cfg->disks_number();
     const unsigned_type min_num_read_write_buffers = (write_buffers_multiple + read_buffers_multiple) * ndisks;
     const unsigned_type nmaxbuckets = m - min_num_read_write_buffers;
-    const unsigned_type lognbuckets = static_cast<unsigned_type>(log2(double(nmaxbuckets)));
+    const unsigned_type lognbuckets = log2_floor(nmaxbuckets);
     const unsigned_type nbuckets = 1 << lognbuckets;
-    const unsigned_type est_bucket_size = div_and_round_up((last - first) / int64(nbuckets),
-                                                           int64(block_type::size)); //in blocks
+    const unsigned_type est_bucket_size = div_ceil((last - first) / nbuckets, block_type::size);      //in blocks
 
     if (m < min_num_read_write_buffers + 2 || nbuckets < 2) {
         STXXL_ERRMSG("stxxl::stable_ksort: Not enough memory. Blocks available: " << m <<
                      ", required for r/w buffers: " << min_num_read_write_buffers <<
                      ", required for buckets: 2, nbuckets: " << nbuckets);
-        abort();
+        throw bad_parameter("stxxl::stable_ksort(): INSUFFICIENT MEMORY provided, please increase parameter 'M'");
     }
     STXXL_VERBOSE_STABLE_KSORT("Elements to sort: " << (last - first));
     STXXL_VERBOSE_STABLE_KSORT("Number of buckets has to be reduced from " << nmaxbuckets << " to " << nbuckets);
@@ -261,7 +269,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
 
     int64 * bucket_sizes = new int64[nbuckets];
 
-    disk_queues::get_instance()->set_priority_op(disk_queue::WRITE);
+    disk_queues::get_instance()->set_priority_op(request_queue::WRITE);
 
     stable_ksort_local::distribute(
         bucket_bids,
@@ -297,7 +305,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
         }
         // here we can increase write_buffers_multiple_b knowing max(bucket_sizes[i])
         // ... and decrease max_bucket_size_bl
-        const int_type max_bucket_size_act_bl = div_and_round_up(max_bucket_size_act, block_type::size);
+        const int_type max_bucket_size_act_bl = div_ceil(max_bucket_size_act, block_type::size);
         STXXL_VERBOSE_STABLE_KSORT("Reducing required number of required blocks per bucket from " <<
                                    max_bucket_size_bl << " to " << max_bucket_size_act_bl);
         max_bucket_size_rec = max_bucket_size_act;
@@ -308,7 +316,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
         typedef buf_ostream<block_type, typename ExtIterator_::bids_container_iterator> buf_ostream_type;
         buf_ostream_type out(first.bid(), nwrite_buffers_bs);
 
-        disk_queues::get_instance()->set_priority_op(disk_queue::READ);
+        disk_queues::get_instance()->set_priority_op(request_queue::READ);
 
         if (first.block_offset())
         {
@@ -332,20 +340,18 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
         type_key_ * refs2 = new type_key_[max_bucket_size_rec];
 
         // submit reading first 2 buckets (Peter's scheme)
-        unsigned_type nbucket_blocks = div_and_round_up(bucket_sizes[0], block_type::size);
+        unsigned_type nbucket_blocks = div_ceil(bucket_sizes[0], block_type::size);
         for (i = 0; i < nbucket_blocks; i++)
             reqs1[i] = blocks1[i].read(bucket_bids[0][i]);
 
 
-        nbucket_blocks = div_and_round_up(bucket_sizes[1], block_type::size);
+        nbucket_blocks = div_ceil(bucket_sizes[1], block_type::size);
         for (i = 0; i < nbucket_blocks; i++)
             reqs2[i] = blocks2[i].read(bucket_bids[1][i]);
 
 
         key_type offset = 0;
-        const unsigned log_k1 =
-            (std::max)(static_cast<unsigned>(ceil(log2(double(
-                                                           max_bucket_size_rec * sizeof(type_key_) / STXXL_L2_SIZE)))), 1U);
+        const unsigned log_k1 = STXXL_MAX<unsigned>(log2_ceil(max_bucket_size_rec * sizeof(type_key_) / STXXL_L2_SIZE), 1);
         unsigned_type k1 = 1 << log_k1;
         int_type * bucket1 = new int_type[k1];
 
@@ -357,10 +363,8 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
 
         for (unsigned_type k = 0; k < nbuckets; k++)
         {
-            nbucket_blocks = div_and_round_up(bucket_sizes[k], block_type::size);
-            const unsigned log_k1_k =
-                (std::max)(static_cast<unsigned>(ceil(log2(
-                                                          double(bucket_sizes[k] * sizeof(type_key_) / STXXL_L2_SIZE)))), 1U);
+            nbucket_blocks = div_ceil(bucket_sizes[k], block_type::size);
+            const unsigned log_k1_k = STXXL_MAX<unsigned>(log2_ceil(bucket_sizes[k] * sizeof(type_key_) / STXXL_L2_SIZE), 1);
             assert(log_k1_k <= log_k1);
             k1 = 1 << log_k1_k;
             std::fill(bucket1, bucket1 + k1, 0);
@@ -393,7 +397,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
                 type_key_ * cEnd = refs2 + bucket1[i];
                 type_key_ * dEnd = refs1 + bucket1[i];
 
-                const unsigned log_k2 = static_cast<unsigned>(log2(double(bucket1[i]))) - 1;        // adaptive bucket size
+                const unsigned log_k2 = log2_floor(bucket1[i]) - 1;        // adaptive bucket size
                 const unsigned_type k2 = 1 << log_k2;
                 int_type * bucket2 = new int_type[k2];
                 const unsigned shift2 = shift1 - log_k2;
@@ -416,7 +420,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
             const unsigned_type bucket2submit = k + 2;
             if (bucket2submit < nbuckets)
             {
-                nbucket_blocks = div_and_round_up(bucket_sizes[bucket2submit], block_type::size);
+                nbucket_blocks = div_ceil(bucket_sizes[bucket2submit], block_type::size);
                 for (i = 0; i < nbucket_blocks; i++)
                     reqs1[i] = blocks1[i].read(bucket_bids[bucket2submit][i]);
             }
@@ -456,7 +460,7 @@ void stable_ksort(ExtIterator_ first, ExtIterator_ last, unsigned_type M)
                   dist_end - begin << " s");
     STXXL_VERBOSE("Time in I/O wait(ds): " << io_wait_after_d << " s");
     STXXL_VERBOSE(*stats::get_instance());
-    UNUSED(begin + dist_end + io_wait_after_d);
+    STXXL_UNUSED(begin + dist_end + end + io_wait_after_d);
 }
 
 //! \}
