@@ -4,6 +4,7 @@
  *  Part of the STXXL. See http://stxxl.sourceforge.net
  *
  *  Copyright (C) 2004-2005 Roman Dementiev <dementiev@ira.uka.de>
+ *  Copyright (C) 2009 Andreas Beckmann <beckmann@cs.uni-frankfurt.de>
  *
  *  Distributed under the Boost Software License, Version 1.0.
  *  (See accompanying file LICENSE_1_0.txt or copy at
@@ -28,11 +29,10 @@ class logger : public singleton<logger>
 
     std::ofstream log_stream_;
     std::ofstream errlog_stream_;
+    std::ofstream * waitlog_stream_;
 
-    inline logger() :
-        log_stream_("stxxl.log"),
-        errlog_stream_("stxxl.errlog")
-    { }
+    logger();
+    ~logger();
 
 public:
     inline std::ofstream & log_stream()
@@ -44,6 +44,11 @@ public:
     {
         return errlog_stream_;
     }
+
+    inline std::ofstream * waitlog_stream()
+    {
+        return waitlog_stream_;
+    }
 };
 
 __STXXL_END_NAMESPACE
